#!/usr/bin/env node
import { promisify } from "util";
import cp from "child_process";
import path from "path";
import fs from "fs";
import chalk from "chalk";

// convert libs to promises
const exec = promisify(cp.exec);
const rm = promisify(fs.rm);

const art = `
              ^^^^^^^^^^^:     ~5B&@@@#P7                              JGGJ      :???????????: ^????
           7GY!?5555555555?:  J@@@#PPB@@@P. ..  .:.   .:.      .::.    P@@5 .:.   7YYY555YYY7   ?555
         7B@@@&Y!?Y555555555?^G@@G.   Y@@@!7&#BG&&&G?P&&&BY. J#&&&&BY: 5@@#B&&&B7  . ~555^ .    7555
      .7B@@@@@@@&5!?5555555555JJ#Y    7@@@7?@@@575@@@B?J&@@Y ^Y?7?B@@P 5@@&Y7Y@@@~   ^555^      7555
    .7B@@@@@@@@@&5: ^?55555555JJ#G.   Y@@@!?@@B  .&@@!  5@@P.YBBGPB@@#.5@@P   #@@7   ^555^      7555
  .?#@@@@@@@@@&Y:     ^?555Y?:7@@@#GG#@@@5 ?@@G  :&@@~  5@@57@@&!~P@@#.5@@5  .#@@7   ^555^      7555
 !B@@@@@@@@@&5^         ^J?:   ^YB&@@&B5!  7B5^  .B##~  J##Y JB##BYB#G.Y##Y   G##!   ^YY?.      !5YY
`;

// Split the art into lines
const lines = art.split("\n");

// Print each line using chalk
lines.forEach((line) => {
  console.log(chalk.hex("#F08619")(line));
});
console.log(chalk.yellow("Omahti CMS Web Starter initiating... \n"));

if (process.argv.length < 3) {
  console.log(chalk.red("ERROR: Missing project name"));
  console.log("You have to provide a name to this new project.");
  console.log("For example:");
  console.log(chalk.underline("npx omahti-cms-web-starter new-project"));
  console.log(chalk.yellow("Exiting... \n"));

  process.exit(1);
}

const projectName = process.argv[2];
const currentPath = process.cwd();
const projectPath = path.join(currentPath, projectName);

const git_repo = "https://github.com/OmahTI-UGM/omahti-cms-web-starter.git";

// create project directory
if (fs.existsSync(projectPath)) {
  console.log(chalk.red("ERROR: Project already exists"));
  console.log(
    `The file ${projectName} already exist in the current directory, please give it another name.`,
  );
  console.log(chalk.yellow("Exiting... \n"));

  process.exit(1);
} else {
  fs.mkdirSync(projectPath);
}

try {
  console.log(chalk.whiteBright("Cloning the boilerplate..."));

  // clone the repo into the project folder -> creates the new boilerplate
  await exec(`git clone ${git_repo} ${projectName} --quiet`);

  console.log(chalk.whiteBright("Cleaning up..."));

  // remove my git history
  const rmGit = rm(path.join(projectPath, ".git"), {
    recursive: true,
    force: true,
  });
  // remove the installation file
  const rmBin = rm(path.join(projectPath, "bin"), {
    recursive: true,
    force: true,
  });
  await Promise.all([rmGit, rmBin]);

  process.chdir(projectPath);

  console.log(chalk.whiteBright("Installing dependencies..."));
  await exec("npm install");

  console.log(chalk.green("The installation is done!"));
  console.log("You can now run your app with:");
  console.log(`    cd ${projectName}`);
  console.log(`    npm run dev`);
  console.log(chalk.yellow("Fuck again!!! - Hafiiz, 2024"));
} catch (error) {
  // clean up in case of error, so the user does not have to do it manually
  fs.rmSync(projectPath, { recursive: true, force: true });
  console.log(error);
}
